@@ -34,6 +34,7 @@
 
 #include "mmcli.h"
 #include "mmcli-common.h"
+#include "mmcli-output.h"
 
 #define PROGRAM_NAME    "mmcli"
 #define PROGRAM_VERSION PACKAGE_VERSION
@@ -43,12 +44,17 @@ static GMainLoop *loop;
 static GCancellable *cancellable;
 
 /* Context */
+static gboolean output_keyvalue_flag;
 static gboolean verbose_flag;
 static gboolean version_flag;
 static gboolean async_flag;
 static gint timeout = 30; /* by default, use 30s for all operations */
 
 static GOptionEntry main_entries[] = {
+    { "output-keyvalue", 'K', 0, G_OPTION_ARG_NONE, &output_keyvalue_flag,
+      "Run action with machine-friendly key-value output",
+      NULL
+    },
     { "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose_flag,
       "Run action with verbose logs",
       NULL
@@ -132,7 +138,7 @@ print_version_and_exit (void)
 {
     g_print ("\n"
              PROGRAM_NAME " " PROGRAM_VERSION "\n"
-             "Copyright (2011 - 2016) Aleksander Morgado\n"
+             "Copyright (2011 - 2019) Aleksander Morgado\n"
              "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl-2.0.html>\n"
              "This is free software: you are free to change and redistribute it.\n"
              "There is NO WARRANTY, to the extent permitted by law.\n"
@@ -184,8 +190,6 @@ main (gint argc, gchar **argv)
 
     setlocale (LC_ALL, "");
 
-    g_type_init ();
-
     /* Setup option context, process it and destroy it */
     context = g_option_context_new ("- Control and monitor the ModemManager");
     g_option_context_add_group (context,
@@ -232,6 +236,16 @@ main (gint argc, gchar **argv)
     if (verbose_flag)
         g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_MASK, log_handler, NULL);
 
+    /* Setup output */
+    if (output_keyvalue_flag) {
+        if (verbose_flag) {
+            g_printerr ("error: cannot set verbose output in keyvalue output type\n");
+            exit (EXIT_FAILURE);
+        }
+        mmcli_output_set (MMC_OUTPUT_TYPE_KEYVALUE);
+    } else
+        mmcli_output_set (MMC_OUTPUT_TYPE_HUMAN);
+
     /* Setup signals */
     signal (SIGINT, signals_handler);
     signal (SIGHUP, signals_handler);
@@ -394,7 +408,7 @@ main (gint argc, gchar **argv)
     } else if (mmcli_modem_messaging_options_enabled ()) {
         mmcli_modem_messaging_shutdown ();
     } else if (mmcli_modem_voice_options_enabled ()) {
-           mmcli_modem_voice_shutdown ();
+        mmcli_modem_voice_shutdown ();
     } else if (mmcli_modem_time_options_enabled ()) {
         mmcli_modem_time_shutdown ();
     } else if (mmcli_modem_firmware_options_enabled ()) {
