@@ -34,6 +34,10 @@
 /* This headers are not exported within ModemManager */
 # include <mm-manager.h>
 # include <mm-object.h>
+# include <mm-sim.h>
+# include <mm-sms.h>
+# include <mm-call.h>
+# include <mm-bearer.h>
 # include <mm-modem.h>
 # include <mm-modem-3gpp.h>
 # include <mm-modem-3gpp-ussd.h>
@@ -62,6 +66,7 @@
 #include <mm-call-properties.h>
 #include <mm-bearer-properties.h>
 #include <mm-bearer-ip-config.h>
+#include <mm-bearer-stats.h>
 #include <mm-location-common.h>
 #include <mm-location-3gpp.h>
 #include <mm-location-gps-raw.h>
@@ -70,8 +75,11 @@
 #include <mm-unlock-retries.h>
 #include <mm-network-timezone.h>
 #include <mm-firmware-properties.h>
+#include <mm-firmware-update-settings.h>
 #include <mm-cdma-manual-activation-properties.h>
 #include <mm-signal.h>
+#include <mm-kernel-event-properties.h>
+#include <mm-pco.h>
 
 /* generated */
 #include <mm-errors-types.h>
