@@ -19,23 +19,32 @@
 #include <config.h>
 #include <glib.h>
 
+#include "mm-filter.h"
+
 #if !defined(MM_DIST_VERSION)
 # define MM_DIST_VERSION VERSION
 #endif
 
-void mm_context_init (gint argc,
+void mm_context_init (gint    argc,
                       gchar **argv);
 
-gboolean     mm_context_get_debug               (void);
-const gchar *mm_context_get_log_level           (void);
-const gchar *mm_context_get_log_file            (void);
-gboolean     mm_context_get_timestamps          (void);
-gboolean     mm_context_get_relative_timestamps (void);
+gboolean     mm_context_get_debug                 (void);
+const gchar *mm_context_get_initial_kernel_events (void);
+gboolean     mm_context_get_no_auto_scan          (void);
+
+/* Filter support */
+MMFilterRule mm_context_get_filter_policy (void);
+
+/* Logging support */
+const gchar *mm_context_get_log_level               (void);
+const gchar *mm_context_get_log_file                (void);
+gboolean     mm_context_get_log_journal             (void);
+gboolean     mm_context_get_log_timestamps          (void);
+gboolean     mm_context_get_log_relative_timestamps (void);
 
 /* Testing support */
-gboolean     mm_context_get_test_session        (void);
-gboolean     mm_context_get_test_no_auto_scan   (void);
-gboolean     mm_context_get_test_enable         (void);
-const gchar *mm_context_get_test_plugin_dir     (void);
+gboolean     mm_context_get_test_session    (void);
+gboolean     mm_context_get_test_enable     (void);
+const gchar *mm_context_get_test_plugin_dir (void);
 
 #endif /* MM_CONTEXT_H */
