@@ -287,16 +287,27 @@ add_generic_version (MMBaseModem               *self,
                      MMFirmwareUpdateSettings  *update_settings,
                      GError                   **error)
 {
-    const gchar *revision;
+    const gchar *firmware_revision;
+    const gchar *carrier_revision;
+    gchar       *combined;
 
-    revision = mm_iface_modem_get_revision (MM_IFACE_MODEM (self));
-    if (!revision) {
+    firmware_revision = mm_iface_modem_get_revision (MM_IFACE_MODEM (self));
+    if (!firmware_revision) {
         g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
                      "Unknown revision");
         return FALSE;
     }
 
-    mm_firmware_update_settings_set_version (update_settings, revision);
+    mm_iface_modem_get_carrier_config (MM_IFACE_MODEM (self), NULL, &carrier_revision);
+
+    if (!carrier_revision) {
+        mm_firmware_update_settings_set_version (update_settings, firmware_revision);
+        return TRUE;
+    }
+
+    combined = g_strdup_printf ("%s - %s", firmware_revision, carrier_revision);
+    mm_firmware_update_settings_set_version (update_settings, combined);
+    g_free (combined);
     return TRUE;
 }
 
@@ -311,6 +322,7 @@ add_generic_device_ids (MMBaseModem               *self,
     GPtrArray   *ids;
     MMPort      *primary = NULL;
     const gchar *subsystem;
+    const gchar *aux;
 
     vid = mm_base_modem_get_vendor_id (self);
     pid = mm_base_modem_get_product_id (self);
@@ -334,7 +346,16 @@ add_generic_device_ids (MMBaseModem               *self,
         return FALSE;
     }
 
+    mm_iface_modem_get_carrier_config (MM_IFACE_MODEM (self), &aux, NULL);
+
     ids = g_ptr_array_new_with_free_func ((GDestroyNotify)g_free);
+    if (aux) {
+        gchar *carrier;
+
+        carrier = g_ascii_strup (aux, -1);
+        g_ptr_array_add (ids, g_strdup_printf ("USB\\VID_%04X&PID_%04X&REV_%04X&CARRIER_%s", vid, pid, rid, carrier));
+        g_free (carrier);
+    }
     g_ptr_array_add (ids, g_strdup_printf ("USB\\VID_%04X&PID_%04X&REV_%04X", vid, pid, rid));
     g_ptr_array_add (ids, g_strdup_printf ("USB\\VID_%04X&PID_%04X", vid, pid));
     g_ptr_array_add (ids, g_strdup_printf ("USB\\VID_%04X", vid));
